//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	command.c
//
// Revision:	1.0
//
// DESCRIPTION:	TMCC data command processor routines
//
//*****************************************************************************


/* include files */
#include <htc.h>
#include "main.h"
#include "globals.h"
#include "command.h"
#include "lights.h"


/* locals */
static uchar debounce_BELL;
static uchar debounce_FRONT_COUPLER;
static uchar debounce_REAR_COUPLER;
static uchar debounce_TWO_KEY;
static uchar debounce_AUX_2;
static schar relative_speed32;

static uchar horn_repeat_timer;
static uchar bell_repeat_timer;
static uchar two_key_repeat_timer;



//******************************************************************************
//
// FUNCTION NAME:	Command_Process()
//
// DESCRIPTION:   	5msec timer routines
//
//******************************************************************************

void Command_Process(void)
{

	// Horn offset control
	if (horn_repeat_timer)
		HORN_OFFSET = TRUE;
	else
		HORN_OFFSET = FALSE;

	// SHORT Horn offset control
	if (!horn_repeat_timer)
	{
		if (two_key_repeat_timer)
			HORN_OFFSET = TRUE;
		else
			HORN_OFFSET = FALSE;
	}

	// Bell offset control
	if (bell_repeat_timer)
		BELL_OFFSET = TRUE;
	else
		BELL_OFFSET = FALSE;


	// run timers
	Service_Command_Debounce_Timers();
}


//******************************************************************************
//
// FUNCTION NAME:	Command_Decoder()
//
// DESCRIPTION:   	Main command decoder
//
//******************************************************************************

void Command_Decoder(uchar Com9bit, uchar cmnd)
{

	//  (TMCC1) 9th bit set processing path
	if (Com9bit)
	{
		// TM1 command parser
		TMCC1_Decode(cmnd);
	}

	//  (TMCC2) 9th bit clear processing path (future expansion)
	else
	{
		// TM2 command parser
		TMCC2_Decode(cmnd);
	}
}


//******************************************************************************
//
// FUNCTION NAME:	TMCC1_Decode
//
// DESCRIPTION:		Decode TMCC1 commands
//
//******************************************************************************

void TMCC1_Decode(uchar cmnd)
{
	if (cmnd == TMCC1_ZERO)
	{
	}

	else if (cmnd == TMCC1_TOGGLE_DIR)
	{
		tcb_data.forward_reverse ^= 1;
	}

	else if (cmnd == TMCC1_REVERSE)
	{
		tcb_data.forward_reverse = REVERSE;
	}

	else if (cmnd == TMCC1_ALT_FORWARD)
	{
		tcb_data.forward_reverse = FORWARD;
	}

	else if (cmnd == TMCC1_HORN_1)
	{
		// Horn commands come in every 100msec on average
		// set timer to 250 msec to allow for command drop-out
		// 5msec time base, so 250msec / 5msec = a count of 50
		horn_repeat_timer = 50;
	}

	else if (cmnd == TMCC1_BELL)
	{
		// Bell commands come in every 100msec on average
		// set timer to 250 msec to allow for command drop-out
		// 5msec time base, so 250msec / 5msec = a count of 50
		bell_repeat_timer = 50;
	}

	else if (cmnd == TMCC1_TWO)
	{
		if (debounce_TWO_KEY)
			cmnd = TMCC1_NOP;
		else
			// 500 msec of horn pulse will activate dialogs
			two_key_repeat_timer = 100;

		debounce_TWO_KEY = DEBOUNCE_TIME;
	}


	else if (cmnd == TMCC1_AUX2)
	{
		// only once, otherwise NOP
		if (debounce_AUX_2)
			cmnd = TMCC1_NOP;

		debounce_AUX_2 = DEBOUNCE_TIME;
	}


	// examples only...

	else if (cmnd == TMCC1_FRONT_COUPLER)
	{
		// only once, otherwise NOP
		if (debounce_FRONT_COUPLER)
			cmnd = TMCC1_NOP;

		debounce_FRONT_COUPLER = DEBOUNCE_TIME;
	}

	else if (cmnd == TMCC1_REAR_COUPLER)
	{
		// only once, otherwise NOP
		if (debounce_REAR_COUPLER)
			cmnd = TMCC1_NOP;

		debounce_REAR_COUPLER = DEBOUNCE_TIME;
	}

	// end examples


	else if ((cmnd >= TMCC1_REL_SPEED_MIN) && (cmnd <= TMCC1_REL_SPEED_MAX))
	{
		relative_speed32 += (cmnd - TMCC1_REL_SPEED_BASE);

		// limit checks
		if (relative_speed32 < 1)
			relative_speed32 = 0;

		else if (relative_speed32 > 31)
			relative_speed32 = 31;
	}

	else if ((cmnd & 0xE0) == TMCC1_ABS_SPEED_ZERO)
	{
		// maintain tracking for 32 speed step for real time selection switch
		relative_speed32 = cmnd & 0x1F;
	}

	// monitor speed step 32
	if (!relative_speed32)
		tcb_data.motion_active = TRUE;
	else
		tcb_data.motion_active = FALSE;




	// command decoders
	Light_Command_Decoder(cmnd);
}


//******************************************************************************
//
// FUNCTION NAME:	TMCC2_Decode
//
// DESCRIPTION:		Decode TMCC2 commands
//
//******************************************************************************

void TMCC2_Decode(uchar cmnd)
{

	// left this code hook in for future expansion

	// monitor speed step 200
	if (cmnd <= TMCC2_SPEED_MAX)
	{
		if (!cmnd)
			tcb_data.motion_active = TRUE;
		else
			tcb_data.motion_active = FALSE;
	}
}



//******************************************************************************
//
// FUNCTION NAME:	Service_Command_Debounce_Timers()
//
// DESCRIPTION:		update command processor debouce timers
//
//******************************************************************************

void Service_Command_Debounce_Timers(void)
{

	if (debounce_BELL)
		debounce_BELL--;

	if (debounce_FRONT_COUPLER)
		debounce_FRONT_COUPLER--;

	if (debounce_REAR_COUPLER)
		debounce_REAR_COUPLER--;

	if (debounce_TWO_KEY)
		debounce_TWO_KEY--;

	if (debounce_AUX_2)
		debounce_AUX_2--;

	if (horn_repeat_timer)
		horn_repeat_timer--;

	if (two_key_repeat_timer)
		two_key_repeat_timer--;

	if (bell_repeat_timer)
		bell_repeat_timer--;
}



