//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	lights.c
//
// Revision:	1.0
//
// DESCRIPTION: Lighting support routines
//
//*****************************************************************************


/* include files */
#include <htc.h>
#include "main.h"
#include "globals.h"
#include "eeprom.h"
#include "command.h"
#include "lights.h"


/* locals */
static uchar LampFast;
static uchar CabinLampState;

static uchar dimmer_timer;
static uint  horn_timer;
static uint  ditch_timer;



//******************************************************************************
//
// FUNCTION NAME:	Init_Lights()
//
// DESCRIPTION:		Initialize LIGHT Controller
//					
//******************************************************************************

void Init_Lights(void)
{

	// init light states
	LampFast = TRUE;
	CabinLampState = COACH_LIGHTS_BRITE;

	// H/W - turn off all lights
	LED_TWO = OFF;					// PWM'd output, set to OFF
	LED_ONE = LED_THREE = OFF;		// 2 LEDs, Ditch Lamps, set OFF

}


//**************************************************************************
//
// FUNCTION NAME:	Light_Process()
//
// DESCRIPTION:		Called out of MAIN() every 1 ms to control lighting 
//					outputs.  The actual functionality depends on the 
//					type of locomotive lighting scheme.
//
//***************************************************************************

void Light_Process(void)
{

	CabinLightCtl();				// Coach Cabin Light control
	DitchLightCtl();				// Ditch Light control


	// update PWM, Ditch Lamp, timers, etc
	Service_Lamp_Timers();
}



//******************************************************************************
//
// FUNCTION NAME:	Lighting_Command_Decoder()
//
// DESCRIPTION:   	TMCC1 command decoder
//
//******************************************************************************

void Light_Command_Decoder(uchar cmnd)
{

	// AUX2 Key
	if (cmnd == TMCC1_AUX2)
	{
		LampFast ^= 1;
	}

	else if (cmnd == TMCC1_AUX2_OFF)
	{
		// implicit lights OFF command, deactivate lamps
		LampFast = FALSE;
	}

	// special AUX2 ON Key
	else if (cmnd == TMCC1_AUX2_ON)
	{
		// implicit lights ON command, activate lamps
		LampFast = TRUE;
	}

	else if (cmnd == TMCC1_ZERO)
	{
		tcb_data.motion_active = FALSE;
		CabinLampState = COACH_LIGHTS_BRITE;
	}

	else if (cmnd == TMCC1_EIGHT)
	{
		CabinLampState = OFF;
	}

	else if (cmnd == TMCC1_NINE)
	{
		if (CabinLampState == OFF)
			CabinLampState = COACH_LIGHTS_BRITE;

		else if (CabinLampState == COACH_LIGHTS_BRITE)
			CabinLampState = COACH_LIGHTS_DIM;

		else if (CabinLampState == COACH_LIGHTS_DIM)
			CabinLampState = COACH_LIGHTS_BRITE;
	}


	else if (cmnd == TMCC1_TOGGLE_DIR || cmnd == TMCC1_ALT_FORWARD || cmnd == TMCC1_REVERSE)
	{
		tcb_data.motion_active = FALSE;
	}

	else if (cmnd == TMCC1_HORN_1 || cmnd == TMCC1_HORN_2)
	{
		// TBD: verify logic - if motion to enable ditch lamps
		if (tcb_data.motion_active) //(check speed commands and determine if motion)
			horn_timer = HORN_TIME;
	}

}


//**************************************************************************
//
// FUNCTION NAME:	CabinLightCtl()
//
// DESCRIPTION:		Implements Coach Lamp Control
//
//***************************************************************************

void CabinLightCtl(void)
{

	// comment out "LampFast" if AUX2 is NOT to control lamp
	if (LampFast)
	{
		// change brightness on 10 msec base time
		if (dimmer_timer++ > 3)
		{
			dimmer_timer = 0;

			// fast dim on OFF setting
			if (CabinLampState == 0)
				dimmer_timer = 5;

			// adjust PWM UP to new level
			if (LED_TWO < CabinLampState)
				LED_TWO++;
	
			// adjust PWM DOWN to new level
			else if (LED_TWO > CabinLampState)
				LED_TWO--;
		}
	}

	else
	{
		// adjust PWM fairly quick to OFF w/ shutdown
		if (LED_TWO > 0)
			LED_TWO--;
	}

}


//**************************************************************************
//
// FUNCTION NAME:	DitchLightCtl()
//
// DESCRIPTION:		Implements Ditch lamps functions
//
//***************************************************************************

void DitchLightCtl(void)
{

	// comment out "LampFast" if AUX2 is NOT to control lamp
	if (LampFast)
	{
		// activate ditch lamps while in motion only
		if (tcb_data.motion_active)
		{
			DitchLightMgr();
		}
	}
	
	else
	{
		// Both Ditch LEDs OFF
		LED_ONE = OFF;
		LED_THREE = OFF;
		horn_timer = 0;
	}
}


//**************************************************************************
//
// FUNCTION NAME:	DitchLightMgr()
//
// DESCRIPTION:		Implements Diesel Rear Ditch lamp Port Operation
//
//***************************************************************************

void DitchLightMgr(void)
{
	uchar ditch_default_on = TRUE;


	if (ee_data.ditch_light_flags == DITCH_OFF)
	{
		LED_ONE = OFF;
		LED_THREE = OFF;
	}

	else if (ee_data.ditch_light_flags == DITCH_ON)
	{
		LED_ONE = ON;
		LED_THREE = ON;
	}

	else
	{
		if (ee_data.ditch_light_flags == DITCH_OFF_PULSE_ON)
			ditch_default_on = FALSE;

		if (horn_timer)
		{
			if (ditch_timer > DITCH_TIME)
			{
				LED_ONE = ON;
				LED_THREE = OFF;
			}
			else 
			{
				LED_ONE = OFF;
				LED_THREE = ON;
			}
		}
		else
		{
			ditch_timer = 0;
			if (ditch_default_on)
			{
				LED_ONE = ON;
				LED_THREE = ON;
			}
			else
			{
				LED_ONE = OFF;
				LED_THREE = OFF;
			}
		}
	}
}



//******************************************************************************
//
// FUNCTION NAME:	Service_Lamp_Timers()
//
// DESCRIPTION:		Called every 1ms to service lamp timers
//
//******************************************************************************

void Service_Lamp_Timers(void)
{

	// horn timer for ditch lamps duration
	if (horn_timer)
		horn_timer--;

	// manage ditch lamp oscilation frequency
	if (ditch_timer)
		ditch_timer--;
	else
		ditch_timer = DITCH_PERIOD;
}




