//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	MAIN.c
//
// Revision:	1.0
//
// DESCRIPTION: Initialization code and Main routines
//
//*****************************************************************************


/* include files */
#include <htc.h>
#include "main.h"
#include "globals.h"
#include "eeprom.h"
#include "command.h"
#include "serial.h"
#include "lights.h"


/* Config Word 1 & 2 */
__CONFIG(FCMEN_OFF & IESO_OFF & CLKOUTEN_OFF & BOREN_ON & CPD_OFF & CP_ON & MCLRE_OFF & PWRTE_ON & WDTE_ON & FOSC_INTOSC);
__CONFIG(LVP_OFF & BORV_HI & STVREN_ON & PLLEN_OFF & WRT_ALL);


/* locals */
static volatile uchar Slice_1MS;
static volatile uchar Slice_4MS;
static volatile uchar Slice_5MS;
static volatile uchar TimerState;


//******************************************************************************
//
// FUNCTION NAME:	main()
//
// DESCRIPTION:		task loop
//					
//******************************************************************************

void main(void)
{
	// setup processor
	Init_HW();

	// init subsystems
	Init_TCBData();
	Init_EEData();
	Init_Lights();


	// set conv / cmnd operating mode
	Mode_Detect();

	// Main Task Loop
	TASK_LOOP
	{
		// kick the doggie
		CLRWDT();

		// operating modules management
		if (Slice_1MS)
		{
			// maintain ticks
			di();
			Slice_1MS--;
			ei();

			//////////////////////////
			// 1ms period processes //
			//////////////////////////			
			if (SerialIn())
			{
				// decode TMCC serial data
				Command_Decoder(Com9bit, SerData);
			}

			//////////////////////////
			// 5ms period processes //
			//////////////////////////
			Slice_5MS++;
			if (Slice_5MS == 1)
			{
				Command_Process();			// command module
			}

			else if (Slice_5MS == 2)
			{
				Light_Process();			// lighting module
			}

			else if (Slice_5MS == 3)
			{
			}

			else if (Slice_5MS == 4)
			{
			}

			else if (Slice_5MS == 5)
			{
				Slice_5MS = 0;
			}
		}
	}
}


//******************************************************************************
//
// FUNCTION NAME:	Mode_Detect()
//
// DESCRIPTION:		Detect mode of system operation, hold system
//					Task Loop in stasis until detection complete
//
//******************************************************************************

void Mode_Detect(void)
{
	uchar serial_init = 0;

	// lock on to serial stream
	TASK_LOOP
	{
		// kick the doggie
		CLRWDT();

		if (Slice_1MS)
		{
			// maintain ticks
			di();
			Slice_1MS--;
			ei();
	
			if (SerialIn())
			{
				if (Com9bit && (SerData == TMCC1_NOP))
				{
					// serial data detected, exit search loop upon confirmation
					serial_init++;
					if (serial_init >= 3)
						return;
				}
			}
		}
	}
}


//******************************************************************************
//
// FUNCTION NAME:	interrupt isr()
//
// DESCRIPTION:		Timer 0 Interrupt Routine
//
//******************************************************************************

static void interrupt isr(void)
{
	if (TMR0IF)
	{
		// reset interrupt flag, reload timer
		TMR0IF = 0;
		TMR0 = TMR0_RATE;

		// timebase
		Slice_1MS++;

//jz
RA4 ^= 1;  //toggle output port for each 1ms timer tick (temp code)

	}
}


//******************************************************************************
//
// FUNCTION NAME:	Init_HW()
//
// DESCRIPTION:		setup hardware
//					
//******************************************************************************

void Init_HW(void)
{
	// init ports (1 = input)
	TRISA = 0x28;
	PORTA = 0;
	
	TRISC = 0x20;
	PORTC = 0;

	// No Pullups, Prescaler to TMR0 / 32
	OPTION_REG = 0x84;

	// PLL OFF, 16MHz, OSC CFG internal
	OSCCON = 0x78;

	// A/D inactive
	ADCON0 = 0x00;

	// Right justified, FOSC/32, A/D Ref = VSS, VDD
	ADCON1 = 0xA0;

	// disable A/D converters
	ANSELA = 0x00;
	ANSELC = 0x00;

	// disable alt port mapping
	APFCON0 = 0x00;

	// Timer2 prescaler
	T2CON = 0x06;		// 1:16

	// Timer period full
	PR2 = 0xFF;

	// PWM timer select
	CCPTMRS = 0x00;

	// PWM period OFF
	CCPR2H = CCPR2L = 0;

	// CCPx controls, set STD PWM mode
	CCP2CON = 0x0C;


	// configure USART baud
	SPBRG = BAUD3030;

	// RCSTA = SPEN, RX9, CREN
	RCSTA = 0xD0;

	// TXSTA = OFF
	TXSTA = 0x00;

	// watchdog to 1 second, enabled
	WDTCON = 0x15;

	// kick the doggie
	CLRWDT();

	// Configure Timer at 0.5 msec, Enable Intr
	TMR0 = TMR0_RATE;
	T0IE = 1;
	GIE = 1;
}



